# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdio.h>
   # include <stdlib.h>
   # include <dirent.h>
   # include <fcntl.h>
   # include <unistd.h>

   # include "block.h"
   # include "command_line.h"
   # include "cwrite.h"
   # include "failure.h"
   # include "history.h"
   # include "io.h"
   # include "key.h"
   # include "message.h"
   # include "mycurses.h"
   # include "status_line.h"
   # include "string.h"
# endif
# if GLOBAL
   char * _dir_name;
   char * _entry_name;
   bool pwd_dir;
# endif
# if CONST
   # define ENTRY_SIZE 80
# endif

void entry_init()
   { _dir_name = (char *) calloc(1, ENTRY_SIZE);
     _entry_name = (char *) calloc(1, ENTRY_SIZE);
     return;
   }
void entry_split()
   {    // -- splits _entry_line string --
     int user_line_len = string_len(_entry_line);
     //status_line_vprintf("%d", user_line_len);
     char * _user_line_fix = _entry_line + user_line_len;
     while ( _user_line_fix >= _entry_line )
        { if ( * _user_line_fix == '/' )
             { break;
             }
          _user_line_fix--;
        }
     char * _user_line_tmp = _entry_line;
     char * _dir_name_tmp = _dir_name;
     if ( * _user_line_fix == '/' )
        { while ( _user_line_tmp <= _user_line_fix )
             { * _dir_name_tmp++ = * _user_line_tmp++;
             }
        }
     * _dir_name_tmp = 0;
     char * _entry_name_tmp = _entry_name;
     while ( * _user_line_tmp )
        { * _entry_name_tmp++ = * _user_line_tmp++;
        }
     * _entry_name_tmp = 0;
     //status_line_vprintf("%s|%s", _dir_name, _entry_name);
     return;
   }
int  entry_select(const struct dirent * _dirent)
   { if ( _dirent->d_type != DT_DIR && _dirent->d_type != DT_REG )
        { if ( _dirent->d_type != DT_UNKNOWN )
             { return 0;
             }
        }
     if ( _dirent->d_name[0] == '.' && ! _dirent->d_name[1] )
         { return 0;
         }
     if ( ! pwd_dir && _dirent->d_name[0] == '.' && _dirent->d_name[1] == '.' )
        { return 0;
        }
     if ( ! * _entry_name && _dirent->d_name[0] == '.' && _dirent->d_name[1] != '.' )
        { return 0;
        }
     if ( ! line_match((char *)_dirent->d_name, _entry_name) )
        { return 0;
        }
     return 1;
   }

int  entry_complete()
   {    // -- uses _entry_line string --
     DIR * _dir = 0;
     struct dirent ** __namelist;
     int entry_cardinal = 0;
     int entry_ordinal;
     int key = CTRL_I;
     do
        { switch(key)
             { case CTRL_I:
                  if ( entry_cardinal )
                     { string_copy(_entry_line_keep, _entry_line);
                     }
                  entry_split();
                  if ( * _dir_name )
                     { pwd_dir = 0;
                     }
                  else
                     { _dir_name = getcwd(0, 0);
                       pwd_dir = 1;
                     }
                  if ( ! _dir_name )
                     { status_line_print(_message[66]);
                       return CTRL_A;
                     }
                  if ( _dir )
                     { io_closedir(_dir);
                     }
                  _dir = opendir(_dir_name);
                  if ( ! _dir )
                     { status_line_vprintf((char *)"%s: open failed", _dir_name);
                       return CTRL_A;
                     }
                  if ( pwd_dir )
                     { status_line_vprintf((char *)"%s", _dir_name);
                     }
                  else
                     { status_line_print(_message[43]);
                     }
                  entry_cardinal = scandir(_dir_name, & __namelist, entry_select, alphasort);
                  //status_line_vprintf("entry_cardinal = %d", entry_cardinal);
                  if ( ! entry_cardinal )
                     { return CTRL_A;
                     }
                  entry_ordinal = -1;
                  key = KEY_DOWN;
                  continue;
               case KEY_UP:
               case CTRL_P:
                  if ( entry_ordinal <= 0 )
                     { //command_line_stop_print("%s", _entry_line);
                       return CTRL_A;
                     }
                  entry_ordinal--;
                  if ( ! pwd_dir )
                     { if ( __namelist[entry_ordinal]->d_type == DT_DIR )
                          { command_line_stop_print((char *)"%s%s/", _dir_name, __namelist[entry_ordinal]->d_name);
                          }
                       else
                          { command_line_stop_print((char *)"%s%s", _dir_name, __namelist[entry_ordinal]->d_name);
                          }
                     }
                  else
                     { if ( __namelist[entry_ordinal]->d_type == DT_DIR )
                          { command_line_stop_print((char *)"%s/", __namelist[entry_ordinal]->d_name);
                          }
                       else
                          { command_line_stop_print((char *)"%s", __namelist[entry_ordinal]->d_name);
                          }
                     }
                  break;
               case KEY_DOWN:
               case CTRL_N:
                  //status_line_vprintf("%d", entry_cardinal);
                  if ( entry_ordinal == entry_cardinal - 1 )
                     { break;
                     }
                  else
                     { entry_ordinal++;
                     }
                  if ( ! pwd_dir )
                     { if ( __namelist[entry_ordinal]->d_type == DT_DIR )
                          { command_line_stop_print((char *)"%s%s/", _dir_name, __namelist[entry_ordinal]->d_name);
                          }
                       else
                          { command_line_stop_print((char *)"%s%s", _dir_name, __namelist[entry_ordinal]->d_name);
                          }
                     }
                  else
                     { if ( __namelist[entry_ordinal]->d_type == DT_DIR )
                          { command_line_stop_print((char *)"%s/", __namelist[entry_ordinal]->d_name);
                          }
                       else
                          { command_line_stop_print((char *)"%s", __namelist[entry_ordinal]->d_name);
                          }
                     }
                  break;
               default:
                  return key;
             }
          key = curses_getch();
        }
     while ( key && key != CTRL_M && key != CTRL_J);

     return key;
   }
