# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdio.h>
   # include <stdlib.h>
   # include <fcntl.h>
   # include <unistd.h>
   # include <errno.h>

   # include "cursor.h"
   # include "command_line.h"
   # include "cwrite.h"
   # include "datachain.h"
   # include "dataline.h"
   # include "datasystem.h"
   # include "file.h"
   # include "header_line.h"
   # include "io.h"
   # include "key.h"
   # include "message.h"
   # include "preferences.h"
   # include "screen.h"
   # include "status_line.h"
   # include "string.h"
# endif
# if PROTO
   void file_set_ro();
# endif

void file_new()
   {   // -- request datasystem --
     datasystem_new();
     datasystem_set();
     datasystem_init(dataline_start_size);
     //_datasystem->dataline_max_size = dataline_extend_size;

     * _status_register |= TYPE_MODE;

        // -- redraw screen --
     screen_clear();
     header_line_refresh();
     status_line_refresh();
     cursor_move(* _cursor_y, * _cursor_x);

     return;
   }
int  file_open()
   { char * _filename = _entry_line;
     bool flag_ro = 0;
     int fd = open(_filename, O_RDWR);
     if ( fd == -1 )
        { fd = open(_filename, O_RDONLY);
          if ( fd == -1 )
             { return FILE_OPEN_FAILURE;
             }
          flag_ro = 1;
        }
     int eof = io_lseek(fd, 0, SEEK_END);
     char * _file = (char *) calloc(1, eof+1);
     io_lseek(fd, 0, SEEK_SET);
     io_read(fd, _file, eof);
     io_close(fd);

     char * _file_point = _file;
     int line_size;
     int line_size_max = dataline_start_size;
     while ( * _file_point )
        { line_size = 0;
          while ( * _file_point && * _file_point != CTRL_J )
             { if ( * _file_point < ' ' && * _file_point != CTRL_I )
                  { return FILE_OPEN_CTRL_CHAR;
                  }
               if ( * _file_point == 127 )
                  { return FILE_OPEN_BS_CHAR;
                  }
               if ( * _file_point == CTRL_I )
                  { for (int i = 0; i < _preferences->tab_value; i++)
                       { line_size++;
                       }
                    _file_point++;
                  }
               else
                  { line_size++;
                    _file_point++;
                  }
             }
          if ( * _file_point )
             { _file_point++;
             }
          if ( line_size > line_size_max )
             { line_size_max = line_size;
             }
        }

     datasystem_new();
     datasystem_set();
     datasystem_init(line_size_max);
     //_datasystem->dataline_max_size = line_size_max;
     # ifdef TEST_FILE_OPEN
        curses_endwin();
        printf("line_size_max = %d\n", line_size_max);
        exit(-1);
     # endif
     string_copy(_datasystem->_filename, _filename);
     Datachain * _datachain = * __datachain_first;
     _file_point = _file;
     char * _dataline;
     int tab_value = _preferences->tab_value;
        // -- alloc +1, critical to clear screen --
     while ( * _file_point )
       { _datachain->_line = (char *) calloc(1, line_size_max + 1);
         _dataline = _datachain->_line;
         _memset(_dataline, ' ', line_size_max + 1);
          line_size = 0;
          while ( * _file_point && * _file_point != CTRL_J )
             { if ( * _file_point == CTRL_I )
                 { line_size += tab_value;
                   for ( int i = 0; i < tab_value; i++ )
                      { * _dataline++ = ' ';
                      }
                   _file_point++;
                  }
               else
                  { line_size++;
                    * _dataline++ = * _file_point++;
                  }
             }
         _datachain->size = line_size;
          if ( * _file_point )
             { _file_point++;
               * __datachain = _datachain;
               _datachain = (Datachain *) _datachain_insert(_datasystem, * __datachain);
             }
       }
     free(_file);
     _datasystem->read_only = flag_ro;
     file_set_ro();

     return FILE_OPEN_SUCCESS;
   }
void file_close()
   { Datachain * _datachain = _datasystem->_datachain_first;
     Datachain * _datachain_next;

     while ( _datachain )
        {  free(_datachain->_line);
          _datachain_next = _datachain->_next;
           free(_datachain);
         _datachain = _datachain_next;
        }
     * _datasystem->_filename = 0;
     _datasystem->status_register = 0;
     if ( datasystem_ordinal == datasystem_count )
        { datasystem_shift_left();
        }
     else
        { datasystem_defrag();
        }
     datasystem_count--;
     if ( ! datasystem_count )
        { screen_clear();
          screen_cursor();

          return_request = 1;
        }
     return;
   }
bool file_save()
   { int fd = open(_datasystem->_filename, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
     if ( fd == -1 )
        { switch (errno)
             { case EACCES:
                  status_line_print(_message[47]);
                  break;
               case EISDIR:
                  status_line_print(_message[48]);
                  break;
               default:
                  status_line_print(_message[39]);
                  break;
             }
          return 0;
        }
     Datachain * _datachain = * __datachain_first;
     char * _dataline;
     while ( _datachain && _datachain->_next )
        { _dataline = _datachain->_line;
           * (_dataline + _datachain->size) = CTRL_J;
           io_write(fd, _dataline, _datachain->size + 1);
           * (_dataline + _datachain->size) = ' ';
          _datachain = _datachain->_next;
        }

        // -- write last line --
     if ( _datachain && _datachain->size )
        { _dataline = _datachain->_line;
          * (_dataline + _datachain->size) = CTRL_J;
          io_write(fd, _dataline, _datachain->size + 1);
          //io_write(fd, _dataline, _datachain->size);
          * (_dataline + _datachain->size) = ' ';
        }
     io_close(fd);

     * _status_register &= ~MODIFY_STATE;

     return 1;  
   }
void file_set_ro()
   { if ( line_match(_datasystem->_filename, (char *) "Makefile") )
        { _datasystem->read_only = 1;
        }
     return;
   }

int file_open_ordinal(char * _filename)
   { for (int i = DATASYSTEM_FIRST; i <= datasystem_count; i++)
        { if ( string_compare(_filename, _datasystem_pointer[i]->_filename) )
             { return i;
             }
        }
     return -1;
   }
