# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdlib.h>

   # include "block.h"
   # include "color_theme.h"
   # include "command_line.h"
   # include "cursor.h"
   # include "cwrite.h"
   # include "dataline.h"
   # include "datasystem.h"
   # include "message.h"
   # include "mycurses.h"
   # include "screen.h"
   # include "scroll.h"
   # include "search.h"
   # include "string.h"
# endif
# if GLOBAL
   char * _find_str;
   char * _replace_str;

   int search_mode;
   int search_submode;

   int find_len;
   int replace_len;
# endif
# if PROTO
   bool dataline_locate(Datachain * _datachain, int position);
   bool dataline_find(Datachain * _datachain, int position);
   void dataline_replace(Datachain * _datachain, int position);
# endif

void search_init()
   { _find_str    = (char *) calloc(1, screen_cols);
     _replace_str = (char *) calloc(1, screen_cols);

     return;
   }
int search_execute()
   { //Datachain * _datachain;
     Datachain * _datachain_begin, * _datachain_end;
     int i, c;
     int match = 0, line_match;
     int start_position;
     int user_request = SEARCH_REPLACE_YES;
     int cursor_y = * _cursor_y;

     if ( search_submode == SEARCH_START )
        { if ( * _status_register & DETACH_STATE )
             { _datachain_begin = * __datachain_detach_start;
               _datachain_end = * __datachain_detach_end;
             }
          else
             { _datachain_begin = * __datachain_first;
               _datachain_end = * __datachain_last;
             }
          start_position = 0;
        }
     else
        { _datachain_begin = * __datachain;
          start_position = * _dataline_offset;
          if ( * _status_register & DETACH_STATE )
             { _datachain_end = * __datachain_detach_end;
             }
          else
             { _datachain_end = * __datachain_last;
             }
       }

     find_len = string_len(_find_str);
     if ( search_mode & SEARCH_REPLACE )
        { replace_len = string_len(_replace_str);
        }
     cursor_hide();
     Datachain * _datachain = _datachain_begin;
     do
        { if ( _datachain->invisible )
             { continue;
             }
          line_match = 0;
          for ( i = start_position; i < _datachain->size - find_len + 1; i++ )
             { if ( dataline_locate(_datachain, i) )
                  { if ( search_mode & SEARCH_FIND )
                       { if ( ! dataline_find(_datachain, i) )
                            { continue;
                            }
                        }
                    match++;
                    //if ( match == 1 )
                    //   { * _cursor_y = screen_top;
                    //   }
                    line_match++;
                    if ( line_match == 1 )
                       { * __datachain = _datachain;
                         if ( * _cursor_y > visible_lines_up(_datachain) + screen_top - 1)
                            { * _cursor_y = visible_lines_up(_datachain) + screen_top - 1;
                            }
                       }
                    * _dataline_offset = i + find_len;
                    if ( * _dataline_offset + screen_left < screen_right )
                       { * _cursor_x = * _dataline_offset + screen_left;
                       }
                    else
                       { * _cursor_x = screen_right;
                       }
                    screen_redraw();
                    //curses_mvchgat(* _cursor_y, i + screen_left, find_len, A_NORMAL, search_color_pair, 0);
                    curses_mvchgat(* _cursor_y, * _cursor_x - find_len, find_len, A_NORMAL, search_color_pair, 0);
                    curses_touchline(stdscr, * _cursor_y, 1);

                    if ( search_mode & SEARCH_REPLACE )
                       { if ( user_request != SEARCH_REPLACE_ALL )
                            { command_line_clear();
                              command_line_print(_message[20], _find_str, _replace_str);
                              command_line_stop = string_len(_message[20]) - 4 + find_len + replace_len;
                              command_line_size = command_line_stop;
                              command_line_redraw();

                              user_request = command_line_revolve(_search_rstr, user_request, 3);
                            }
                         switch (user_request)
                            { case -1:
                                 screen_redraw();
                                 cursor_show();
                                 return -1;
                              case SEARCH_REPLACE_NO:
                                 break;
                              case SEARCH_REPLACE_YES:
                              case SEARCH_REPLACE_ALL:
                                 dataline_replace(_datachain, i);
                            }
                         if ( * _dataline_offset + screen_left < screen_right )
                            { * _cursor_x = * _dataline_offset + screen_left;
                            }
                         else
                            { * _cursor_x = screen_right;
                            }
                         screen_redraw();
                       }
                    else
                       { do
                            { c = curses_getch();
                              if ( c == 0 )
                                 { screen_redraw();
                                   cursor_show();
                                   return -1;
                                 }
                            }
                         while ( c != KEY_DOWN );
                       }
                  }
             }

          start_position = 0;
          if ( * _cursor_y < screen_bottom )
             { (* _cursor_y)++;
             }
        }
     while ( (_datachain = _datachain->_next) && (_datachain != _datachain_end->_next) );

     if ( ! match )
        { * _cursor_y = cursor_y;
        }
     if ( * _cursor_y > visible_lines_up(* __datachain) + screen_top - 1)
        { * _cursor_y = visible_lines_up(* __datachain) + screen_top - 1;
        }
     screen_redraw();
     cursor_show();

     return match;
   }

bool dataline_locate(Datachain * _datachain, int position)
   { if ( line_match(_datachain->_line + position, _find_str) )
        { return 1;
        }
     return 0;
   }
bool dataline_find(Datachain * _datachain, int position)
   { char * _dataline = _datachain->_line;
     bool prev_c_valid = 1;
     bool next_c_valid = 1;

     //if ( ! position )
     //   { return 0;
     //   }
     int prev_c = ' ';
     if ( position )
        { prev_c = * (_dataline + position - 1);
        }
     int next_c = * (_dataline + position + find_len);

     if ( prev_c >= '0' && prev_c <= '9' )
        { prev_c_valid = 0;
        }
     if ( prev_c >= 'a' && prev_c <= 'z' )
        { prev_c_valid = 0;
        }
     if ( prev_c >= 'A' && prev_c <= 'Z' )
        { prev_c_valid = 0;
        }
     if ( prev_c == '_' )
        { prev_c_valid = 0;
        }
     if ( next_c >= '0' && next_c <= '9' )
        { next_c_valid = 0;
        }
     if ( next_c >= 'a' && next_c <= 'z' )
        { next_c_valid = 0;
        }
     if ( next_c >= 'A' && next_c <= 'Z' )
        { next_c_valid = 0;
        }
     if ( next_c == '_' )
        { next_c_valid = 0;
        }

     return prev_c_valid & next_c_valid;
   }
void dataline_replace(Datachain * _datachain, int position)
   { int status_register = * _status_register;
     * _status_register |= INS_MODE;

     * _dataline_offset -= find_len;
     dataline_shift_left(_datachain, find_len);

     char * _replace_point = _replace_str;
     while ( * _replace_point )
        { dataline_fill(* _replace_point++);
        }

     * _status_register = status_register;
     * _status_register |= MODIFY_STATE;

     return;
   }
